/***************************************************************************************
					ERROR HANDLING ROUTINES
***************************************************************************************/
#include "predict.h"
#include "ray.h"

void ERROR_Message(char* s1, char* s2)
{
  printf("Error, %s: %s", s1, s2);
  cc=getchar();
  exit(1); 
}
	/* routines for controlling the debug.txt file */
void CloseDF()
{ 	
	if (gfp_Debug != NULL)
		fclose(gfp_Debug);
}
void SaveDF()
{
	if (gfp_Debug != NULL)
		{ fclose(gfp_Debug); gfp_Debug = fopen("Debug.txt", "a+"); }
}
void OpenDF()
{	gfp_Debug = fopen("Debug.txt", "w");  }
/************/

void ERROR_Check_Verts_Planar(long* piWallVert, short iNumVert)
{
}

int Vert_CompareX(const void* pVert1, const void* pVert2)
{
	VERTEX_TYPE*	pV1 = (VERTEX_TYPE*) pVert1;
	VERTEX_TYPE*	pV2 = (VERTEX_TYPE*) pVert2;

	if (pV1->fx < pV2->fx)
		return(-1);
	else if (pV1->fx > pV2->fx)
		return(1);
	else
		return(0);
}

void ERROR_Check_For_Duplicate_Vertices(VERTEX_TYPE* pVert, long iNumVert, char* s)
{
	VERTEX_TYPE*	pV = (VERTEX_TYPE*) calloc(iNumVert, sizeof(VERTEX_TYPE));
	VERTEX_TYPE*	pVsave = pV;
	VERTEX_TYPE		v1, v2;
	long			i, j, k, l;
	float			fx;

	for(i=0; i < iNumVert; i++)
		*(pV+i) = *(pVert+i);

	qsort((void**) pV, iNumVert, sizeof(VERTEX_TYPE), (int (*)(const void*, const void*))(Vert_CompareX));

	for(i=0; i < iNumVert; i++)
	{
		fx = pV->fx;
		l = 0;
		while (fx == (pV+l+1)->fx && ((i+l) < iNumVert))	l++;
		for(j=0; j<=l; j++)
		{
		  for(k=j+1; k<=l; k++)
		    {
		      v1 = *(pV+j);  v2 = *(pV+k);
		      if (Macro_PointsEqual3D(v1, v2))
			{
			  printf("Duplicate points found in file %s.\n", s);
			  printf("Point: %f %f %f", v1.fx, v1.fy, v1.fz);
			 
			  /* exit(1); */
			}
		    }
		}
		pV = (pV + l + 1); i = i + l;
	}
	
	free(pVsave);					
}

/***************************************************************************************
	Check_Tri_Data:
		Used to test that the Triangle data structures have been correctly created.
***************************************************************************************/ 
void Check_Tri_Data(void* apTri[], long* piNum_Tri, short QUERY)
{
  int 	i, j;
  VERTEX_TYPE      *pV;
  EDGE_TYPE        *pE;
  TRIANGLE_TYPE    *pT;
  
  printf("HERE AT CHECK TRIANGLE DATA:\n\n");
  printf("Num points: %i\n", *piNum_Tri);
  printf("     edges: %i\n", *(piNum_Tri+1));
  printf(" triangles: %i\n\n", *(piNum_Tri+2));

	pV = (VERTEX_TYPE *) apTri[0];
    j = 0;
    for(i=1; i<=*piNum_Tri; i++)
	{
	  j++;
	  printf("%i %lf %lf\n", i, pV->fx, pV->fy);
	  pV++;
	  if (j==20)
	    {
	      j=0;
	      cc = getchar();
	    }
	}
  printf("END POINTS\n");
  cc = getchar();  

  j = 0;  
  pE = (EDGE_TYPE*) apTri[1];
  for(i=1; i<=*(piNum_Tri+1); i++)
    {
      j++;
      printf("%li %li %li %li %li\n", i, pE->iPoint[0],pE->iPoint[1],pE->iTri[0],pE->iTri[1]);
      pE++;
      if (j==20)
	{
	  j=0;
	  cc = getchar();
	}
    }
  printf("END EDGES\n");
  cc = getchar();
  j = 0;
  pT = (TRIANGLE_TYPE*) apTri[2];
  for(i=1; i<=*(piNum_Tri+2); i++)
    {
      j++;
      printf("%li %li %li %li %li %li %li\n", i, pT->iPoint[0],pT->iPoint[1],pT->iPoint[2],pT->iEdge[0],pT->iEdge[1],pT->iEdge[2]);
      pT++;
      if (j==20)
	{
	  j=0;
	  cc = getchar();
	}
    }
  printf("END TRIANGLES\n");
  cc = getchar();
}

void Print_Triangle(long iTri)
{
	TRIANGLE_TYPE*		pTri = (TRIANGLE_TYPE*) gpObst_Tri[2] + iTri;
	gpsVert = (VERTEX_TYPE*) gpObst_Tri[0];
	gpsEdge = (EDGE_TYPE*) gpObst_Tri[1];

	if (iTri < 0)	return;

	printf("Triangle points:\n");
	printf("iEdge0 %li    ", pTri->iEdge[0]);
	Print_Point(gpsVert[(gpsEdge+(pTri->iEdge[0]))->iPoint[0]]);
	printf("iEdge0 %li    ", pTri->iEdge[0]);
	Print_Point(gpsVert[(gpsEdge+(pTri->iEdge[0]))->iPoint[1]]);
	printf("iEdge1 %li    ", pTri->iEdge[1]);
	Print_Point(gpsVert[(gpsEdge+(pTri->iEdge[1]))->iPoint[0]]);
	printf("iEdge1 %li    ", pTri->iEdge[1]);
	Print_Point(gpsVert[(gpsEdge+(pTri->iEdge[1]))->iPoint[1]]);
	printf("iEdge2 %li    \n", pTri->iEdge[2]);
}
void Print_Point(POINT_TYPE sP)
{
	printf("Point: %f %f %f\n", sP.fx, sP.fy, sP.fz);
}
void Print_Vector(VECTOR_TYPE sV)
{
	printf("Vector: %f %f %f\n", sV.i, sV.j, sV.k);
}
void fPrint_Point(FILE* fp, POINT_TYPE sP)
{
	fprintf(fp, "Point: %f %f %f\n", sP.fx, sP.fy, sP.fz);
}
void fPrint_Vector(FILE* fp, VECTOR_TYPE sV)
{
	fprintf(fp, "Vector: %f %f %f\n", sV.i, sV.j, sV.k);
}
void Print_Plane(PLANE_TYPE sPl) 
{
	printf("	PLANE NORMAL: "); Print_Vector(sPl.sN);
	printf("	PLANE POINT: "); Print_Point(sPl.sP);
}
void fPrint_Plane(FILE* fp, PLANE_TYPE sPl) 
{
	fprintf(fp, "	PLANE NORMAL: "); fPrint_Vector(fp, sPl.sN);
	fprintf(fp, "	PLANE POINT: "); fPrint_Point(fp, sPl.sP);
}
void Print_Ray(RAY_TYPE sRay)
{
	printf("   PRINTING RAY %i\n", giNumRayStack);
	printf("	sRayVect: "); Print_Vector(sRay.sRayVect);
	printf(" 	asTriVect[0]: "); Print_Vector(sRay.asTriVect[0]);
	printf(" 	asTriVect[1]: "); Print_Vector(sRay.asTriVect[1]);
	printf(" 	asTriVect[2]: "); Print_Vector(sRay.asTriVect[2]);
	printf(" 	asTriP[0]: "); Print_Point(sRay.asTriP[0]);
	printf(" 	asTriP[1]: "); Print_Point(sRay.asTriP[1]);
	printf(" 	asTriP[2]: "); Print_Point(sRay.asTriP[2]);
	printf("	sVirtTransLoc: "); Print_Point(sRay.sVirtTransLoc);
	printf("	sLKP: "); Print_Point(sRay.sLKP);
	printf("	iTriID: %li", sRay.iTriID); 
	printf("	iEdgeID: %li\n", sRay.iEdgeID); 
	printf("	fAtten: %f", sRay.fAtten);
	printf("	fDist: %f\n", Macro_SqrtDist(sRay.sLKP, sRay.sVirtTransLoc)); 
	printf("	cNumEvent: %c", sRay.cNumEvent); 
	printf("	cPrevEvent: %c", sRay.cPrevious); 
	printf("   END\n");
}
void fPrint_Ray(FILE* fp, RAY_TYPE sRay)
{
	fprintf(fp, "  PRINTING RAY %i\n", giNumRayStack);
	fprintf(fp, "	sRayVect: "); fPrint_Vector(fp, sRay.sRayVect);
	fprintf(fp, " 	asTriVect[0]: "); fPrint_Vector(fp, sRay.asTriVect[0]);
	fprintf(fp, " 	asTriVect[1]: "); fPrint_Vector(fp, sRay.asTriVect[1]);
	fprintf(fp, " 	asTriVect[2]: "); fPrint_Vector(fp, sRay.asTriVect[2]);
	fprintf(fp, " 	asTriP[0]: "); fPrint_Point(fp, sRay.asTriP[0]);
	fprintf(fp, " 	asTriP[1]: "); fPrint_Point(fp, sRay.asTriP[1]);
	fprintf(fp, " 	asTriP[2]: "); fPrint_Point(fp, sRay.asTriP[2]);
	fprintf(fp, "	sVirtTransLoc: "); fPrint_Point(fp, sRay.sVirtTransLoc);
	fprintf(fp, "	sLKP: "); fPrint_Point(fp, sRay.sLKP);
	fprintf(fp, "	iTriID: %li", sRay.iTriID); 
	fprintf(fp, "	iEdgeID: %li\n", sRay.iEdgeID); 
	fprintf(fp, "	fAtten: %f", sRay.fAtten);
	fprintf(fp, "	fDist: %f\n", Macro_SqrtDist(sRay.sLKP, sRay.sVirtTransLoc)); 
	fprintf(fp, "	cNumEvent: %c", sRay.cNumEvent); 
	fprintf(fp, "	cPrevEvent: %c", sRay.cPrevious); 
	fprintf(fp, "  END\n");
}
