
#include "predict.h"

typedef struct
{
	short	iNumVert;
	long*	piVert;
	short	iR, iG, iB;	/* for color information; 0-255 */
	short	iA;			/* transparent to opaque; 0-255 */
} FACE_TYPE;

void Geomview_Output_OFF(char* sFileName, VERTEX_TYPE* pV, long iV, FACE_TYPE* pF, long iF)
{
	FILE*	fpOut = fopen(sFileName, "w");
	long	i, j;
	short	iNum;
	long*	pVert;

	if (fpOut==NULL)    ERROR_Message("Error opening file: ", sFileName); 

	fprintf(fpOut, "OFF\n");
	fprintf(fpOut, "%li %li 0\n", iV+2, iF);

	for(i=0; i<iV; i++)
		fprintf(fpOut, "%f %f %f\n", (pV+i)->fx, (pV+i)->fy, (pV+i)->fz); 
			
			/* bounding region size */
	fprintf(fpOut, "%f %f %f\n", gasBR[0].fx, gasBR[0].fy, gasBR[0].fz); 
	fprintf(fpOut, "%f %f %f\n", gasBR[1].fx, gasBR[1].fy, gasBR[1].fz); 

	for(i=0; i<iF; i++)
	{
		iNum = (pF+i)->iNumVert;
		pVert = (pF+i)->piVert;
		fprintf(fpOut, "%i ", iNum);
		for(j=0; j<iNum; j++)
			fprintf(fpOut, "%li ", *(pVert+j));
		fprintf(fpOut, "%hi %hi %hi %hi\n", (pF+i)->iR, (pF+i)->iG, (pF+i)->iB, (pF+i)->iA);
	}
	
	fclose(fpOut);
}			

void Free_Faces(FACE_TYPE* pF, long iF)
{
	long	i;

	for(i=0; i<iF; i++)
		free((pF+i)->piVert);
	free(pF);
}

void Geomview_Out(char* s)
{
	VERTEX_TYPE*		pVert, *pV;
	FACE_TYPE*			pFace, *pF;
	long				iNumV, iNumF, iQP_ID;
	long				i, j;
	double				fPathLoss_Total;

	if (strcmp(s, "Wall_Data") == 0)
	{
		pVert = gpsObst_Vertices; iNumV = giNumObstVert;
		pFace = (FACE_TYPE*) calloc(giNumObstWalls, sizeof(FACE_TYPE));
		pF = pFace; iNumF = giNumObstWalls;		
		for(i=0; i<iNumF; i++){
			pF->iNumVert = (gpsObst_Walls+i)->iNumVert;
			pF->piVert = (long*) calloc(pF->iNumVert, sizeof(long));
			for(j=0; j<pF->iNumVert; j++)
				*(pF->piVert + j) = *((gpsObst_Walls+i)->piVert + j);
					/* color info */
			pF->iA=160; pF->iR=160; pF->iG=160; pF->iB=220;
			pF++;
		}
		Geomview_Output_OFF("Walls.OFF", pVert, iNumV, pFace, iNumF);
		Free_Faces(pFace, iNumF);
	}
	else if (strcmp(s, "Obst_Tri") == 0)
	{
		pVert = (VERTEX_TYPE*) calloc(gaiNum_Obst[0], sizeof(VERTEX_TYPE));
		pV = (VERTEX_TYPE*) gpObst_Tri[0]; iNumV = gaiNum_Obst[0];
 
if (giCL_Terrain) { pV = gpsTerrain; iNumV = giNumTerr; }
			/**** Setup the vertice data ***/
		for(i=0; i < iNumV; i++)
		  {
		    (pVert+i)->fx = (pV+i)->fx; 
		    (pVert+i)->fy = (pV+i)->fy; 
		    (pVert+i)->fz = (pV+i)->fz * 3; if ((pV+i)->fz > 30) (pVert+i)->fz = (float) 0;
		  }
		/***************************************/
			/***** Face Data *********/
		pFace = (FACE_TYPE*) calloc(gaiNum_Obst[1], sizeof(FACE_TYPE));
		pF = pFace; iNumF = gaiNum_Obst[1];
		for(i=0; i<iNumF; i++){
			pF->iNumVert = 2;
			pF->piVert = (long*) calloc(pF->iNumVert, sizeof(long));
			for(j=0; j<pF->iNumVert; j++) 
				*(pF->piVert + j) = (((EDGE_TYPE*) gpObst_Tri[1])+i)->iPoint[j];
			if ((((EDGE_TYPE*) gpObst_Tri[1])+i)->iWallType > 0) {
				pF->iR = 200; pF->iA = 255;
			}
			else {
					pF->iR = 255; pF->iB = 255; pF->iG = 255; pF->iA = 255;
			}
			pF++;
		}
			/********* Create the Geomview file and free the allocated space ******/
		Geomview_Output_OFF("ObstTri.OFF", pVert, iNumV, pFace, iNumF);
		free(pVert);
		Free_Faces(pFace, iNumF);
	}
	else if (strcmp(s, "QP_Tri") == 0)
	{
		pVert = (VERTEX_TYPE*) calloc(gaiNumQP[0], sizeof(VERTEX_TYPE));
		pV = (VERTEX_TYPE*) gpQP_Tri[0]; iNumV = gaiNumQP[0];
			/**** Setup the vertice data ***/
		for(i=0; i < iNumV; i++)
		  {
		    (pVert+i)->fx = (pV+i)->fx; 
		    (pVert+i)->fy = (pV+i)->fy; 
		    (pVert+i)->fz = (pV+i)->fz - 2;
		  }
		/***************************************/
			/***** Face Data *********/
		pFace = (FACE_TYPE*) calloc(gaiNumQP[2], sizeof(FACE_TYPE));
		pF = pFace; iNumF = gaiNumQP[2];
		for(i=0; i<iNumF; i++){
 			pF->iNumVert = 3;
			pF->piVert = (long*) calloc(pF->iNumVert, sizeof(long));
			fPathLoss_Total = (double) 0;
			for(j=0; j<pF->iNumVert; j++) {
				iQP_ID = (((TRIANGLE_TYPE*) gpQP_Tri[2])+i)->iPoint[j];
				*(pF->piVert + j) = iQP_ID;
				if (iQP_ID < giNumQP && (gpsQPoints + iQP_ID)->fBest_Path_Loss > (double) -100)
					fPathLoss_Total = fPathLoss_Total + (gpsQPoints + iQP_ID)->fBest_Path_Loss;
				else
					fPathLoss_Total = fPathLoss_Total + (double) -100;
			}
				/* figure out the region's color */
			if (fPathLoss_Total <= -299)
				pF->iG = (short) 0;
			else
				pF->iG = (short) (255 * (1 - (fPathLoss_Total / -300))) + (short) 20;
			if (pF->iG > 255)   pF->iG = 255;
			pF->iR = 0; pF->iB = 0; pF->iA = 200; 
			 /* */
			pF++;
		}
			/********* Create the Geomview file and free the allocated space ******/
		Geomview_Output_OFF("QPTri.OFF", pVert, iNumV, pFace, iNumF);
		free(pVert);
		Free_Faces(pFace, iNumF);
	}
	else
		ERROR_Message("No match in the Geomview_Out to", s);
}
