
#include "predict.h"
#include "ray.h"


void Check_Ray_Cone(RAY_TYPE sRay, RAY_TYPE* psRay)
{
	long		iTriID, iEdgeID;
	short		iFlag, i;

	if (sRay.iTriID == -1 || sRay.cNumEvent > gsRS.cMaxEvents) 	return;
	
	iFlag = 0;

	iTriID = PointLocation(sRay.sLKP, gpObst_Tri, gaiNum_Obst, sRay.iEdgeID);
	if (iTriID != sRay.iTriID && iTriID != OtherTri(sRay.iTriID, sRay.iEdgeID))	{ 
/*		printf("sRay.sLKP is not in iTriID!\n"); 
		Print_Ray(sRay); 
		printf("New iTriID = %li\n", iTriID); */
		psRay->iTriID = iTriID;
		sRay.iTriID = iTriID;
		iFlag = 1;
	}
	for(i=0; i<3; i++) {
		iTriID = PointLocation(sRay.asTriP[i], gpObst_Tri, gaiNum_Obst, sRay.iEdgeID);
		if (iTriID > 0 && iTriID != sRay.iTriID && iTriID != OtherTri(sRay.iTriID, sRay.iEdgeID)) {  
/*			if (iFlag == 0) 
				{ Print_Ray(sRay); } */
			iFlag=1;
		}
	} 

	iEdgeID = From_Edge(sRay, sRay.sLKP);
	if (iEdgeID > 0 && iEdgeID != sRay.iEdgeID)	{ 
		iFlag = 1; 
/*		printf("sRay.iEdgeID is wrong! iEdgeID %li  sRay.iEdge %li sRay.Tri %li Other1 %li  Other2 %li\n", iEdgeID, sRay.iEdgeID, sRay.iTriID, OtherTri(sRay.iTriID, sRay.iEdgeID), OtherTri(sRay.iTriID, iEdgeID)); */
		psRay->iEdgeID = iEdgeID; 
	}

	/* if (iFlag)	{ printf("Check_Ray FLAG"); cc=getchar(); } */
}

	/* returns a point that is a slightly shifted version of sP1 towards sP2 */
POINT_TYPE	Point_Offset(POINT_TYPE sP1, POINT_TYPE sP2)
{
	POINT_TYPE		sOff = sP1;
	PARA_LINE_TYPE	sLine;
	
	if (Macro_PointsEqual3D(sP1, sP2))		return(sP1);

	sLine = Line_Parametric_Equation3D(sP1, sP2);

	sOff = ParaLine_Calc_Point(sLine.sV, sLine.sP, (float) .005);

	return(sOff);
}
/* The a ray-cone needs to know which edge it would have come from */
	/* assumes that sP is inside of iTriID for ray */
long From_Edge(RAY_TYPE sRay, POINT_TYPE sPIn)
{
	int				i;
	long			iEdgeID, iEdgeRetID = -1, iTriID;
	POINT_TYPE		sEdgePoint1, sEdgePoint2, sP;
 	VERTEX_TYPE*	psVertices = (VERTEX_TYPE*) gpObst_Tri[0];
	EDGE_TYPE*		psEdges = (EDGE_TYPE*) gpObst_Tri[1];
	TRIANGLE_TYPE*	psTri = (TRIANGLE_TYPE*) gpObst_Tri[2];

	sP = ParaLine_Calc_Point(sRay.sRayVect, sPIn, (float) -10000);
	iTriID = sRay.iTriID;
	if (sRay.iTriID < 0)	{ return(sRay.iTriID); }
	i = 0; 
	while (i<3)
	{
		iEdgeID = (psTri+iTriID)->iEdge[i];
		sEdgePoint1 = psVertices[(psEdges+iEdgeID)->iPoint[0]];
		sEdgePoint2 = psVertices[(psEdges+iEdgeID)->iPoint[1]];
		if (Intersect_Segments2D(sPIn, sP, sEdgePoint1, sEdgePoint2))
			{ i = 3; iEdgeRetID = iEdgeID; }
		i++;
	}
		/* must lie right on an edge */
	if (iEdgeRetID == -1 && (sRay.sRayVect.i > .005 || sRay.sRayVect.j > .005)) {		
 		sPIn = ParaLine_Calc_Point(sRay.sRayVect, sPIn, (float) .005);
		iTriID = PointLocation(sPIn, gpObst_Tri, gaiNum_Obst, (psTri+iTriID)->iEdge[0]);
		if (iTriID != sRay.iTriID)	sRay.iTriID = iTriID; 
		iEdgeRetID = From_Edge(sRay, sPIn);
	}

	return(iEdgeRetID);
}
void Print_Trap(TRAP_TYPE sT)
{
	int	i;

	for(i=0; i<5; i++) 
		Print_Plane(sT.sP[i]);
}
	/* sV and sP define the line.  sP2 is assumed to be on the line.  T_Dist is
		the t value needed such that sP2 = sP * sV * t */
float T_Dist(VECTOR_TYPE sV, POINT_TYPE sP, POINT_TYPE sP2)
{
	float	fT;

 	if (Macro_PointsEqual3D(sP, sP2))
		return((float) 0);

	if (Macro_NotZero(sV.i))
		fT = (float) fabs((sP2.fx - sP.fx) / sV.i);
	else if (Macro_NotZero(sV.j))
		fT = (float) fabs((sP2.fy - sP.fy) / sV.j);
	else
		fT = (float) fabs((sP2.fz - sP.fz) / sV.k);

	return(fT);
}
void Print_Ray_Tri(RAY_TRI_TYPE asTri[], short iNum)
{
	int		i, k, j=0;

	printf("  NumTri: %i\n", iNum); 
	for(i=0; i<iNum; i++) {
		for(k=0; k<4; k++)
			Print_Point(asTri[i].asP[k]);
		printf("iEdgeID = %i   iPlane  %i   iActive %i\n", asTri[i].iEdgeID, asTri[i].iPlane, asTri[i].iActive);
		j++;
		/* if (j==3 && i != iNum-1)	{ cc=getchar(); j=0; } */
	}
	/* cc=getchar(); */
}

void Extend_Points(POINT_TYPE* psP1, POINT_TYPE* psP2)
{
	PARA_LINE_TYPE	sLine;

	sLine = Line_Parametric_Equation3D(*psP1, *psP2);

	*psP1 = ParaLine_Calc_Point(sLine.sV, sLine.sP, (float) -1000);
	*psP2 = ParaLine_Calc_Point(sLine.sV, sLine.sP, (float) 1000);

}
	/* the area of the parallelogram squared */
float Triangle_Area(POINT_TYPE asTriP[])
{
	VECTOR_TYPE		sV1, sV2, sV3;
	
		/* compute the area of the ray-cone's end triangle face */
	sV1.i = asTriP[1].fx - asTriP[0].fx; sV1.j = asTriP[1].fy - asTriP[0].fy; sV1.k = asTriP[1].fz - asTriP[0].fz;
	sV2.i = asTriP[2].fx - asTriP[0].fx; sV2.j = asTriP[2].fy - asTriP[0].fy; sV2.k = asTriP[2].fz - asTriP[0].fz;
	sV3.i = (sV1.j * sV2.k) - (sV2.j * sV1.k);
	sV3.j = -1 * ((sV1.i * sV2.k) - (sV2.i * sV1.k));
	sV3.k = (sV1.i * sV2.j) - (sV1.j * sV2.i);
	/* fArea = (float) sqrt(sV3.i * sV3.i + sV3.j * sV3.j + sV3.k * sV3.k) / 2; */
	
	return(sV3.i * sV3.i + sV3.j * sV3.j + sV3.k * sV3.k); 
}

int	All_In_Triangle(POINT_TYPE asP[], POINT_TYPE asP2[], RAY_TYPE sRay, PLANE_TYPE sPlane)
{
	PARA_LINE_TYPE	sLine;
	int				iAll, j;
	TRIANGLE_TYPE*	psTri = (TRIANGLE_TYPE*)	gpObst_Tri[2];

	sLine.sP = sRay.sVirtTransLoc;
	iAll = 1;
	for(j=0; j<4; j++)	{
		sLine = Line_Parametric_Equation3D(sRay.sVirtTransLoc, asP2[j]);
		asP[j] = Line_Plane_Intersection(sLine, sPlane);
		if (!InsideTri(asP[j], *(psTri+sRay.iTriID)))
			{ iAll = 0; j = 4;}
	}

	return(iAll);
}
