/***************************************************************************************
				DATA STRUCTURES SAVE and LOAD ROUTINES
***************************************************************************************/

#include "predict.h"

void Save_TriDS(char* sFileName, void* PET[], long iNum[])
{
  	FILE 			*filePtr;
	long			i;
	VERTEX_TYPE*	pV = (VERTEX_TYPE*) PET[0];
	EDGE_TYPE*		pE = (EDGE_TYPE*) PET[1];
	TRIANGLE_TYPE*  pT = (TRIANGLE_TYPE*) PET[2];
  
  filePtr = fopen(sFileName, "w");
  if (filePtr==NULL)    ERROR_Message("Error opening file for writing: ", sFileName); 

	fprintf(filePtr, "%li %li %li\n", iNum[0], iNum[1], iNum[2]);

	for(i=0; i<iNum[0]; i++)
	{
		fprintf(filePtr, "%f %f\n", pV->fx, pV->fy);
		pV++;
	}

	for(i=0; i<iNum[1]; i++)
	{
		fprintf(filePtr, "%li %li %li %li %hi %hi\n", pE->iPoint[0], pE->iPoint[1], 
													pE->iTri[0], pE->iTri[1],
													pE->iWallType, pE->iThickness);
		pE++;
	}

	for(i=0; i<iNum[2]; i++)
	{
		fprintf(filePtr, "%li %li %li %li %li %li\n", 
							pT->iPoint[0], pT->iPoint[1], pT->iPoint[2],
							pT->iEdge[0], pT->iEdge[1], pT->iEdge[2]);
		pT++;
	}

	fclose(filePtr);
}

void Load_TriDS(char* sFileName, void* apPET[], long* piNumTri)
{
  	FILE 			*filePtr;
	long			i, iNum[3];
	VERTEX_TYPE*	pV;
	EDGE_TYPE*		pE;
	TRIANGLE_TYPE*  pT;
  
  filePtr = fopen(sFileName, "r");
  if (filePtr==NULL)    ERROR_Message("Error opening file for reading: ", sFileName); 

	fscanf(filePtr, "%li %li %li\n", &iNum[0], &iNum[1], &iNum[2]);
		/* allocate memory for storage of the triangle data structure */
	pV = (VERTEX_TYPE*) calloc(iNum[0], sizeof(VERTEX_TYPE));
	pE = (EDGE_TYPE*) calloc(iNum[1], sizeof(EDGE_TYPE));
	pT = (TRIANGLE_TYPE*) calloc(iNum[2], sizeof(TRIANGLE_TYPE));
		/* assign pointers to the triangle data structure pointer array */
	apPET[0] = pV;		*piNumTri = iNum[0];
	apPET[1] = pE; *(piNumTri+1) = iNum[1];
	apPET[2] = pT;	*(piNumTri+2) = iNum[2];

	for(i=0; i<iNum[0]; i++)
	{
		fscanf(filePtr, "%f %f\n", &(pV->fx), &(pV->fy));
		pV->fz = (float) 0;
		pV++;
	}

	for(i=0; i<iNum[1]; i++)
	{
		fscanf(filePtr, "%li %li %li %li %hi %hi\n", &(pE->iPoint[0]), &(pE->iPoint[1]), 
													&(pE->iTri[0]), &(pE->iTri[1]),
													&(pE->iWallType), &(pE->iThickness));
		pE++;
	}

	for(i=0; i<iNum[2]; i++)
	{
		fscanf(filePtr, "%li %li %li %li %li %li\n", 
							&(pT->iPoint[0]), &(pT->iPoint[1]), &(pT->iPoint[2]),
							&(pT->iEdge[0]), &(pT->iEdge[1]), &(pT->iEdge[2]));
		pT++;
	}

	fclose(filePtr);
}

/***************************************************************************************
	Save_Query_Tri:
		Stores the query point triangulation data structure and query point data.

	QP_Tri.dat - is the file where the query point triangulatio DS is stored

	QPdata.dat - is the file where the query point info is stored (pathloss, ...)
***************************************************************************************/   
void Save_Query_Points(QP_TYPE* pQPoints, void* asPET[], long aiNum[])
{
  	long            i, j;
  	FILE           	*fpout, *fp;
	QP_TYPE*		psQP = gpsQPoints;
  	SIGNAL_TYPE*	psSig;
	double			fPath_Loss, fDelay_Spread;
	
	if (giCL_Stats)	fp = fopen("QStats.dat", "w");
		
		/* save triangulation data structure */
	Save_TriDS("QP_Tri.dat", asPET, aiNum); 
		/* create a geomview file for displaying the triangulation */
	if (giCL_Geomview)	Geomview_Out("QP_Tri");

  	fpout = fopen("QPdata.dat", "w");
	  if (fpout==NULL)    ERROR_Message("Error opening file for writing: ", "QPdata.dat"); 
  	fprintf(fpout, "%i\n", giNumQP);

 	for(i=0; i<giNumQP; i++)  
    {
      	fprintf(fpout, "%f %f %f %hi %lf %hi\n", psQP->sLoc.fx, psQP->sLoc.fy, psQP->sLoc.fz,
      			psQP->iBest_Trans_ID, psQP->fBest_Path_Loss, psQP->iNum_Trans_Can_Hear);
		psSig = psQP->psSignal_Data;
  		for(j=0; j < psQP->iNum_Trans_Can_Hear; j++)  
    	{
			if (giCL_Stats && j == psQP->iBest_Trans_ID) {
				fPath_Loss = psSig->fPath_Loss;  
				fDelay_Spread = psSig->fDelay_Spread;
			}
      		fprintf(fpout,"%lf %lf %f %f %hi\n", psSig->fPath_Loss, psSig->fDelay_Spread, 
      							psSig->fShortPath, psSig->fShortStr, psSig->iTrans_ID);
			psSig = psSig->psNext;
	    }
		if (giCL_Stats)	fprintf(fp, "%lf %le\n", fPath_Loss, fDelay_Spread);
      	psQP++;
    }
  	fclose(fpout);
  	if (giCL_Stats)	fclose(fp);
}

QP_TYPE* Load_Query_Points(void* asPET[], long aiNum[])
{
  	int            	i;
  	FILE           	*fpin;
	QP_TYPE*		psQP, *psQPoints;
  	SIGNAL_TYPE*	psSig;

	Load_TriDS("QP_Tri.dat", asPET, aiNum); 

  	fpin = fopen("QPdata.dat", "r");
  		if (fpin==NULL)    ERROR_Message("Error opening file for reading: ", "QPdata.dat"); 
  	fscanf(fpin, "%i/n", &giNumQP);

	psQPoints = (QP_TYPE*) calloc(giNumQP, sizeof(QP_TYPE));
 	psQP = psQPoints;

 	for(i=0; i<giNumQP; i++)  
    {
      	fscanf(fpin, "%f %f %f %hi %lf %hi\n", &(psQP->sLoc.fx), &(psQP->sLoc.fy), 
      			&(psQP->iBest_Trans_ID), &(psQP->fBest_Path_Loss), &(psQP->iNum_Trans_Can_Hear));

		psQP->psSignal_Data = (SIGNAL_TYPE*) calloc(psQP->iNum_Trans_Can_Hear, sizeof(SIGNAL_TYPE));
		psSig = psQP->psSignal_Data;
  		for(i=0; i < psQP->iNum_Trans_Can_Hear; i++)  
    	{
      		fscanf(fpin,"%lf %lf %f %f %hi\n", &(psSig->fPath_Loss), &(psSig->fDelay_Spread), 
      							&(psSig->fShortPath), &(psSig->fShortStr), &(psSig->iTrans_ID));
			if (i < psQP->iNum_Trans_Can_Hear-1)
				psSig->psNext = (psSig+1);
			else
				psSig->psNext = NULL;
			psSig++;
	    }
      	psQP++;
    }
  	fclose(fpin);

	return(psQPoints);
}

